﻿
<System.ComponentModel.ProvideProperty("PropertyName", GetType(Control))>
Public Class ControlMapper
    Inherits System.ComponentModel.Component
    Implements System.ComponentModel.IExtenderProvider

    ''' <summary>
    ''' Dictionary to hold control/property mapping
    ''' </summary>
    ''' <remarks></remarks>
    Dim Mapped As Dictionary(Of Control, String)

    Public Sub New()
        'Instantiate the dictionary
        Mapped = New Dictionary(Of Control, String)
    End Sub

    ''' <summary>
    ''' Get function used by the ProvideProperty attribute
    ''' </summary>
    ''' <param name="myControl"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetPropertyName(ByVal myControl As Control) As String
        'Get property name based on control, return empty string if not found
        If Mapped.ContainsKey(myControl) Then
            Return Mapped.Item(myControl)
        Else
            Return ""
        End If
    End Function

    ''' <summary>
    ''' Set function used by the ProvideProperty attribute
    ''' </summary>
    ''' <param name="myControl"></param>
    ''' <param name="value"></param>
    ''' <remarks></remarks>
    Public Sub SetPropertyName(ByVal myControl As Control, ByVal value As String)
        'Add property/control pair to dictionary
        If Mapped.ContainsKey(myControl) Then
            Mapped.Item(myControl) = value
        Else
            Mapped.Add(myControl, value)
        End If
        'Add event handler for control type
        If myControl.GetType() Is GetType(TextBox) Then
            Dim tb As TextBox = myControl
            AddHandler tb.TextChanged, AddressOf Me.TextChanged
        End If
    End Sub

    ''' <summary>
    ''' The IExtenderProvider interface, returns true if the passed control type can be extended.
    ''' </summary>
    ''' <param name="extendee"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function CanExtend(ByVal extendee As Object) As Boolean Implements System.ComponentModel.IExtenderProvider.CanExtend
        If extendee.GetType Is GetType(TextBox) Then
            Return True
        Else
            Return False
        End If
    End Function

    ''' <summary>
    ''' Used to set the class the form will be updating
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property FormObject As Object

    ''' <summary>
    ''' Internal flag the the form is being filled
    ''' value changes should be ignored when true
    ''' </summary>
    ''' <remarks></remarks>
    Dim Filling As Boolean = False

    ''' <summary>
    ''' Called to initiallize all controls from the values stored in the class
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub FillFromClass()
        'Check for null
        If FormObject IsNot Nothing Then
            Try
                'Set filing flag, use try...finally structure to 
                'ensure the flag is always turned off
                Filling = True
                'Loop through all handled controls
                For Each item In Mapped
                    'Get control on property name from dictionary item
                    Dim Cntl As Control = item.Key
                    Dim PropertyName As String = item.Value
                    'Get value from class 
                    Dim Value As Object
                    'Get reflection property
                    Dim Prop = FormObject.GetType().GetProperty(PropertyName)
                    'Check null 
                    If Prop IsNot Nothing Then
                        'get property value
                        Value = Prop.GetValue(FormObject, Nothing)
                        'Set value according to control type 
                        If Cntl.GetType() Is GetType(TextBox) Then
                            'Set text property for text boxes
                            Dim TB As TextBox = Cntl
                            TB.Text = Value
                            'Done!
                        End If
                    End If
                Next
            Finally
                Filling = False
            End Try
        End If
    End Sub

    Private Sub TextChanged(ByVal sender As Control, ByVal e As EventArgs)
        'Check for DesingMode
        If Not DesignMode Then
            'Check if the control is being filled
            If Not Filling Then
                'Check for null
                If FormObject IsNot Nothing Then
                    'Get the name of the property bound to control
                    Dim PropetyName = GetPropertyName(sender)
                    'Check if found
                    If PropetyName <> "" Then
                        'Get property using reflection
                        Dim Prop = FormObject.GetType().GetProperty(PropetyName)
                        'Check null
                        If Prop IsNot Nothing Then
                            'Get value based on control type 
                            Dim Value As Object
                            If sender.GetType() Is GetType(TextBox) Then
                                'Get text property
                                Value = CType(sender, TextBox).Text
                            Else
                                'Unsupported control type
                                Return
                            End If
                            'Set the value of the class to the new value
                            Prop.SetValue(FormObject, Value, Nothing)
                            'All done!
                        End If
                    End If
                End If
            End If
        End If
    End Sub

End Class
